#!/usr/bin/env python
"""\
Simple g-code streaming script for grbl

Provided as an illustration of the basic communication interface
for grbl. When grbl has finished parsing the g-code block, it will
return an 'ok' or 'error' response. When the planner buffer is full,
grbl will not send a response until the planner buffer clears space.

G02/03 arcs are special exceptions, where they inject short line 
segments directly into the planner. So there may not be a response 
from grbl for the duration of the arc.
"""

import serial
import time
import re
import time
import sys
import argparse
import os

if sys.platform == 'darwin':
    import cv2
    
    def still(outfile):
        retval = vidcap.grab()
        retval, image = vidcap.retrieve()
        cv2.imwrite(outfile, image)
else:
    # raspberry pi

    os.system('/home/pi/code/raspifastcamd/start_camd.sh /home/pi/images/frame%04d.png')

    def still(outfile):
        #os.system('raspistill -o ' + outfile)
        os.system('/home/pi/code/raspifastcamd/do_capture.sh')

botPort = None
    
def directCommand(cmd):
    global botPort
    botPort.write(cmd+'\n')
    
def goHome():
    print "(going home)"
    directCommand("$H")
                  #G0 X{0:0.5f} Y{1:0.5f} Z0.0".format(homea, homeb))

def penDown():
    directCommand("M3")


def penUp():
    directCommand("M5")


def main():
    global botPort

    if sys.platform == 'darwin':
        # open webcam
        vidcap = cv2.VideoCapture()
        vidcap.open(1)
        print "Webcam Open"

    RX_BUFFER_SIZE = 64#128

    # Define command line argument interface
    parser = argparse.ArgumentParser(description='Stream g-code file to grbl. (pySerial and argparse libraries required)')
    parser.add_argument('gcode_file', type=argparse.FileType('r'),
            help='g-code filename to be streamed')
    parser.add_argument('device_file',
            help='serial device path')
    parser.add_argument('outpath',
                        help = 'outpath for image files')

    parser.add_argument('pre_pause', help = 'pause at each position, before taking photo',
                        default=0.5)
    
    parser.add_argument('post_pause', help = 'pause at each position, after taking photo',
                        default=0.5)
    

    args = parser.parse_args()      

    # Open grbl serial port
    # Initialize
    s = serial.Serial(args.device_file,9600)
    botPort = s
    
    # Open g-code file
    f = args.gcode_file
    outpath = args.outpath

    # Wake up grbl
    print "Initializing grbl..."
    s.write("\r\n\r\n")

    # Wait for grbl to initialize and flush startup text in serial input
    time.sleep(2)
    s.flushInput()

    # Stream g-code to grbl
    print "Streaming ", args.gcode_file.name, " to ", args.device_file

    s_count = 0
    r_count = 0
    pic_count = 0

    lines = f.readlines()
    numlines = len(lines)
    l_count = 0

    preroll = 0

    pre_pause = float(args.pre_pause)
    post_pause = float(args.post_pause)
    
    try:
        while True:
            try:

                if l_count >= numlines:
                    break
                
                l = lines[l_count].strip()
                l_count += 1 # Iterate line counter

                # send line to grbl
                s_count += 1
                print 'Sending:', s_count, l,
                s.write(l + '\n')

                # Wait for grbl response with carriage return
                grbl_out = s.readline() 
                r_count += 1
                print ' :', r_count, grbl_out.strip()

                if l_count == 3:
                    # Wait for grbl response with carriage return
                    grbl_out = s.readline()
                    r_count += 1
                    print ' :', r_count, grbl_out.strip()

##                    # Wait for grbl response with carriage return
##                    grbl_out = s.readline()
##                    r_count += 1
##                    print ' :', r_count, grbl_out.strip()

                # sent a move command
                if l[:2] == "G1" or l[:2] == "G0":
                    print "moved",
                    parts = l.split(" ")

                    # target position
                    target_xpos = round(float(parts[1][1:]), 1)
                    target_ypos = round(float(parts[2][1:]), 1)

                    # loop until we hit point
                    there = False
                    doWrite = True

                    while not there:
                        # check for responses
                        grbl_out = s.readline().strip() # Wait for grbl response with carriage return

                        #print grbl_out
                        
                        if grbl_out.find("ok") > 0 or grbl_out.find("k") > 0:
                            #print ".",
                            doWrite = True
                        
                        #print grbl_out
                        elif grbl_out.split(',')[0] == "<Idle":
                            ret_parts = grbl_out.split(",")
                            #print ret_parts
                            xpos = round(float(ret_parts[4][5:]), 1)
                            ypos = round(float(ret_parts[5]),1)
                            #print xpos, ypos,
                            
                            if xpos == target_xpos and ypos == target_ypos:
                                there = True
                            else:
                                doWrite = True

                        # send query
                        if doWrite:
                            s.write('?' + '\n')
                            

                    # arrive at point
                    print "snap"
                    time.sleep(pre_pause) #1.5)
                    outfile = os.path.join(outpath, "test_{0:04d}.png".format(pic_count))            
                    still(outfile)
                    pic_count += 1
                    time.sleep(post_pause) #1.5)
                    
            except (KeyboardInterrupt):
                nb = None
                while nb is not "":
                    nb = raw_input("\ncommand (? for help):")
                    if nb == "q":
                        goHome()
                        break
                    elif nb == "h":
                        goHome()
                    elif nb == "d":
                        penDown()
                    elif nb == "u":
                        penUp()
                    elif nb == "?":
                        print """h to home
    d to put pen down
    u to lift pen up
    q to go home and quit
    type a sentence to pass as gcode command
    """,
                    else:
                        s.write(nb) # send to grbl

                if nb =="q":
                    break

    except (KeyboardInterrupt):
        print "exiting..."
        pass


    time.sleep(3)
    
    # Close file and serial port
    f.close()
    s.close()

    if sys.platform != 'darwin':
        os.system('/home/pi/code/raspifastcamd/stop_camd.sh')



if __name__ == "__main__":
    main()
    
